<?php
/**
 * 2007-2017 Copyright
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author Objective Concept - Copyright
 *  @copyright Copyright
 *  @package   copyright
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

class CopyrightCms extends ObjectModel
{
    public $id_copyright_cms;

    public $date_add;

    public $date_upd;

    public $id_cms;

    public $id_copyright_file;

    public static $definition = array(
        'table' => 'copyright_cms',
        'primary' => 'id_copyright_cms',
        'multilang_shop' => true,
        'fields' => array(
            'id_cms' =>             array('type' => self::TYPE_INT, 'validate' => 'isInt'),
            'id_copyright_file' =>  array('type' => self::TYPE_INT, 'validate' => 'isInt'),
            'date_add' =>           array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'date_upd' =>           array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
        ),
    );

    public function __construct($id = null, $id_lang = null, $id_shop = null)
    {
        self::associateTable();
        parent::__construct($id, $id_lang, $id_shop);
    }

    public static function associateTable()
    {
        if (!Shop::getAssoTable(self::$definition['table'])) {
            Shop::addTableAssociation(self::$definition['table'], array('type' => 'shop'));
        }
    }

    public static function getCms($id_cms)
    {
        if (empty($id_cms)) {
            throw new PrestaShopException('$id_cms should not be empty.');
        }

        self::associateTable();
        return Db::getInstance()->getRow(
            'SELECT * FROM ' . _DB_PREFIX_ . self::$definition['table'] . ' AS cms 
            ' . Shop::addSqlAssociation(self::$definition['table'], 'cms') . '
            WHERE cms.`id_cms` = ' . pSQL($id_cms)
        );
    }

    public static function getNotRegistrationCms()
    {
        self::associateTable();
        CopyrightFile::associateTable();

        return Db::getInstance()->executeS(
            'SELECT * FROM ' . _DB_PREFIX_ . self::$definition['table'] . ' AS cms '
            . Shop::addSqlAssociation(self::$definition['table'], 'cms')
            . ' INNER JOIN ' . _DB_PREFIX_ . CopyrightFile::$definition['table']
            . ' AS cf ON (cf.`id_copyright_file` = cms.`id_copyright_file`) '
            . Shop::addSqlAssociation(CopyrightFile::$definition['table'], 'cf')
            . ' WHERE cf.`is_recurring` = 1 AND cf.`certificate_url` = "";'
        );
    }

    public static function getNotUpdatedCms()
    {
        self::associateTable();
        CopyrightFile::associateTable();

        return Db::getInstance()->executeS(
            'SELECT * FROM ' . _DB_PREFIX_ . self::$definition['table'] . ' AS cms '
            . Shop::addSqlAssociation(self::$definition['table'], 'cms')
            . ' INNER JOIN ' . _DB_PREFIX_ . CopyrightFile::$definition['table']
            . ' AS cf ON (cf.`id_copyright_file` = cms.`id_copyright_file`) '
            . Shop::addSqlAssociation(CopyrightFile::$definition['table'], 'cf')
            . ' WHERE cf.`status` = 0 AND cf.`is_recurring` = 1 AND cf.`certificate_url` = "";'
        );
    }

    public static function getRegisterCms($id_cms)
    {
        self::associateTable();
        CopyrightFile::associateTable();

        return Db::getInstance()->getRow(
            'SELECT * FROM ' . _DB_PREFIX_ . self::$definition['table'] . ' AS cms '
            . Shop::addSqlAssociation(self::$definition['table'], 'cms') . ' INNER JOIN '
            . _DB_PREFIX_ . CopyrightFile::$definition['table']
            . ' AS cf ON (cf.`id_copyright_file` = cms.`id_copyright_file`) '
            . Shop::addSqlAssociation(CopyrightFile::$definition['table'], 'cf')
            . ' WHERE cf.`status` = 1 
                AND cf.`is_recurring` = 1 
                AND cf.`certificate_url` != "" 
                AND cf.`registered_file_url` != ""
                AND cms.`id_cms` = ' . pSQL($id_cms) . ';'
        );
    }

    public static function inProgress($id_cms)
    {
        self::associateTable();
        CopyrightFile::associateTable();

        return Db::getInstance()->getRow(
            'SELECT * FROM ' . _DB_PREFIX_ . self::$definition['table']
            . ' AS cms '
            . Shop::addSqlAssociation(self::$definition['table'], 'cms')
            . ' INNER JOIN ' . _DB_PREFIX_ . CopyrightFile::$definition['table']
            . ' AS cf ON (cf.`id_copyright_file` = cms.`id_copyright_file`) '
            . Shop::addSqlAssociation(CopyrightFile::$definition['table'], 'cf')
            . ' WHERE cf.`is_recurring` = 1 
                AND cf.`certificate_url` = "" 
                AND cf.`registered_file_url` = ""
                AND cms.`id_cms` = ' . pSQL($id_cms) . ';'
        );
    }

    public static function getAllCms()
    {
        self::associateTable();
        CopyrightFile::associateTable();

        return Db::getInstance()->executeS(
            'SELECT * FROM ' . _DB_PREFIX_ . self::$definition['table'] . ' AS cms 
            ' . Shop::addSqlAssociation(self::$definition['table'], 'cms') . '
            INNER JOIN ' . _DB_PREFIX_ . CopyrightFile::$definition['table'] . ' AS cf 
            ON (cf.`id_copyright_file` = cms.`id_copyright_file`)
            ' . Shop::addSqlAssociation(CopyrightFile::$definition['table'], 'cf') .
            ' GROUP BY cms.`' . self::$definition['primary'] . '`'
        );
    }

    public static function createCmsCopyright($id_cms)
    {
        if (empty($id_cms)) {
            throw new PrestaShopException('$id_cms should be an object or id of a product.');
        }

        if (is_numeric($id_cms)) {
            $cms = new CMS($id_cms);
        } elseif (is_object($id_cms)) {
            $cms = $id_cms;
        }

        $context = Context::getContext();
        $copyright_module = Module::getInstanceByName('copyright');

        //create a new user or get a old copyright user
        $copyright_user = CopyrightUser::getCopyrightUser();

        /* Update availible credits */
        $credits = CopyrightConnect::getInstance()
            ->api_auth(
                '/api/Copyright/credits?catalog=' . $copyright_user->cat
                . '&ID_client=' . $copyright_user->id_client
            );

        if ($credits['Copyright'] > 0) {
            $copyright_settings = new CopyrightSettings(CopyrightSettings::getCurrentIdCopyrightSettings());

            /*$copyright_user->copyright_credits = $credits['Copyright'];
            $copyright_user->save();*/
            
            $vars = array(
                'date_upd' => date('Y-m-d H:i:s'),
                'name' => $cms->meta_title[$context->language->id],
                'description' => strip_tags($cms->content[$context->language->id]),
            );

            $protected_document = $copyright_module->renderTemplate('cms_protected_document.tpl', 'admin', $vars);

            $file_name = $cms->id . '.pdf';

            $mpdf = new mPDF('+aCJK');

            $mpdf->allow_charset_conversion = true;
            $mpdf->charset_in = 'utf8';

            $mpdf->setAutoTopMargin = 'stretch';
            $mpdf->SetHTMLHeader(
                '<img style="height: 48px;" alt="" src="'
                . $copyright_module->getViewUri() . 'img/logo-full.png" />
				<img style="float:right; height: 40px;" alt="" src="'
                . $copyright_module->getViewUri() . 'img/prestashop_logo.jpg" />
				<br/>'
            );

            $mpdf->setAutoBottomMargin = 'stretch';
            $mpdf->SetHTMLFooter(
                'The Copyright Module provides a solution to the Prestashop Community 
                to protect her content from being copied by a certain date certified by public notary.'
                . "\n"
                . 'This document includes all information on a Post/Article created. The actual owner is : '
                . $copyright_user->first_name . ' ' . $copyright_user->last_name
            );

            $mpdf->SetTitle($cms->meta_title[$context->language->id]);
            $mpdf->SetSubject(Tools::truncate(strip_tags($cms->content[$context->language->id]), 600));
            $mpdf->WriteHTML($protected_document);
            /* 'S'- return the document as a string. $filename is ignored. */
            $file_output = $mpdf->Output($file_name, 'S');

            $data = array(
                "FileName" 		=> $file_name,
                "FileBase64" 	=> base64_encode($file_output),
                "Description" 	=> Tools::truncate(strip_tags($cms->content[$context->language->id]), 600),
                "Title" 		=> $cms->meta_title[$context->language->id],
                "Catalog" 		=> $copyright_user->cat,
                "ID_client" 	=> $copyright_user->id_client,
                "ID_sector" 	=> $copyright_settings->id_content_category ? $copyright_settings->id_content_category : 6,
            );

            $result = CopyrightConnect::getInstance()->api_auth('/api/Copyright/registercopyright', true, $data);

            if ($result != false) {
                $status = ('ERROR' == $result) ? 0 : 1;

                if ($copyright_cms = self::getCms($cms->id)) {
                    $copyright_file = new CopyrightFile($copyright_cms['id_copyright_file']);
                    $copyright_file->status = $status;
                    $copyright_file->id_deposit = $result;
                    $copyright_file->is_recurring = 1;
                    $copyright_file->save();

                } else {

                    $copyright_file = new CopyrightFile();
                    $copyright_file->status = $status;
                    $copyright_file->id_deposit = $result;
                    $copyright_file->is_recurring = 1;
                    $copyright_file->save();

                    $copyright_product = new self();
                    $copyright_product->id_cms = $cms->id;
                    $copyright_product->id_copyright_file = $copyright_file->id;
                    $copyright_product->save();
                }

            } else {
                return false;
            }

            return true;
        }
    }

    public static function install()
    {
        if (!self::createTables()) {
            return false;
        }

        return true;
    }

    public static function unInstall()
    {
        if (!self::dropTables()) {
            return false;
        }

        return true;
    }

    public static function createTables()
    {
        $tables = array();

        $tables[] = 'CREATE TABLE IF NOT EXISTS ' . _DB_PREFIX_ . self::$definition['table'] . ' (
                        `' . self::$definition['primary'] . '` INT(11) NOT NULL PRIMARY KEY AUTO_INCREMENT,
                        `date_add` TIMESTAMP,
                        `date_upd` TIMESTAMP,
                        `id_cms` INT(11) NOT NULL,
                        `id_copyright_file` INT(11) NOT NULL
                    ) ENGINE=InnoDB CHARACTER SET=UTF8;';

        $tables[] = 'CREATE TABLE IF NOT EXISTS ' . _DB_PREFIX_ . self::$definition['table'] . '_shop (
                        `' . self::$definition['primary'] . '` INT(11) NOT NULL,
                        `id_shop` INT(11) NOT NULL,
                        PRIMARY KEY (`' . self::$definition['primary'] . '`,`id_shop`),
                        KEY (`id_shop`)
                    ) ENGINE=InnoDB CHARACTER SET=UTF8;';

        if (!self::execute($tables)) {
            return false;
        }

        return true;
    }

    public static function dropTables()
    {
        $tables = array();

        $tables[] = 'DROP TABLE IF EXISTS ' . _DB_PREFIX_ . self::$definition['table'] . ';';
        $tables[] = 'DROP TABLE IF EXISTS ' . _DB_PREFIX_ . self::$definition['table'] . '_shop;';

        if (!self::execute($tables)) {
            return false;
        }

        return true;
    }

    public static function execute(array $sqls)
    {
        foreach ($sqls as $sql) {
            if (!Db::getInstance()->execute($sql)) {
                return false;
            }
        }

        return true;
    }
}
