<?php
/**
 * 2007-2017 Copyright
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author Objective Concept - Copyright
 *  @copyright Copyright
 *  @package   copyright
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

class CopyrightFile extends ObjectModel
{
    public $id_copyright_file;

    public $date_add;

    public $date_upd;

    public $id_deposit;

    public $status;

    public $is_recurring;

    public $certificate_url;

    public $registered_file_url;

    public static $definition = array(
        'table' => 'copyright_file',
        'primary' => 'id_copyright_file',
        'multilang_shop' => true,
        'fields' => array(
            'id_deposit' =>             array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'status' =>                 array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'certificate_url' =>        array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'registered_file_url' =>    array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'type' =>                   array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'is_recurring' =>           array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'date_add' =>               array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'date_upd' =>               array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
        ),
    );

    public function __construct($id = null, $id_lang = null, $id_shop = null)
    {
        self::associateTable();
        parent::__construct($id, $id_lang, $id_shop);
    }

    public static function associateTable()
    {
        if (!Shop::getAssoTable(self::$definition['table'])) {
            Shop::addTableAssociation(self::$definition['table'], array('type' => 'shop'));
        }
    }

    public static function install()
    {
        if (!self::createTables()) {
            return false;
        }

        return true;
    }

    public static function unInstall()
    {
        if (!self::dropTables()) {
            return false;
        }

        return true;
    }

    public static function createTables()
    {
        $tables = array();

        $tables[] = 'CREATE TABLE IF NOT EXISTS ' . _DB_PREFIX_ . self::$definition['table'] . ' (
                        `' . self::$definition['primary'] . '` INT(11) NOT NULL PRIMARY KEY AUTO_INCREMENT,
                        `date_add` TIMESTAMP,
                        `date_upd` TIMESTAMP,
                        `id_deposit` CHAR(255) NOT NULL,
                        `status` BOOL NOT NULL,
                        `is_recurring` BOOL NOT NULL,
                        `certificate_url` CHAR(255),
                        `type` CHAR(255),
                        `registered_file_url` CHAR(255)
                    ) ENGINE=InnoDB CHARACTER SET=UTF8;';

        $tables[] = 'CREATE TABLE IF NOT EXISTS ' . _DB_PREFIX_ . self::$definition['table'] . '_shop (
                        `' . self::$definition['primary'] . '` INT(11) NOT NULL,
                        `id_shop` INT(11) NOT NULL,
                        PRIMARY KEY (`' . self::$definition['primary'] . '`,`id_shop`),
                        KEY (`id_shop`)
                    ) ENGINE=InnoDB CHARACTER SET=UTF8;';

        if (!self::execute($tables)) {
            return false;
        }

        return true;
    }

    public static function dropTables()
    {
        $tables = array();

        $tables[] = 'DROP TABLE IF EXISTS ' . _DB_PREFIX_ . self::$definition['table'] . ';';
        $tables[] = 'DROP TABLE IF EXISTS ' . _DB_PREFIX_ . self::$definition['table'] . '_shop;';

        if (!self::execute($tables)) {
            return false;
        }

        return true;
    }

    public static function execute(array $sqls)
    {
        foreach ($sqls as $sql) {
            if (!Db::getInstance()->execute($sql)) {
                return false;
            }
        }
        return true;
    }
}
