<?php
/**
 * 2007-2017 Copyright
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author Objective Concept - Copyright
 *  @copyright Copyright
 *  @package   copyright
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

class CopyrightSettings extends ObjectModel
{

    public $id_copyright_settings;

    public $date_add;

    public $date_upd;

    public $id_product_categories = array();

    public $id_cms_categories;

    public $id_content_category;

    public $available_credits_count = 3;

    public $copyright_notice;

    public $groupBox;

    public $active;

    public static $definition = array(
        'table' => 'copyright_settings',
        'primary' => 'id_copyright_settings',
        'multilang' => true,
        'multilang_shop' => true,
        'fields' => array(
            'id_content_category' => array('type' => self::TYPE_INT, 'validate' => 'isInt'),
            'available_credits_count' => array('type' => self::TYPE_INT, 'validate' => 'isInt'),
            'date_add' => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'date_upd' => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'active' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),

            /* Shop fields */
            'id_product_categories' => array('type' => self::TYPE_STRING, 'shop' => true),
            'id_cms_categories' => array('type' => self::TYPE_STRING, 'shop' => true),

            /* Lang fields */
            'copyright_notice' => array('type' => self::TYPE_HTML, 'lang' => true, 'validate' => 'isCleanHtml'),
        ),
    );

    public function __construct($id = null, $id_lang = null, $id_shop = null)
    {
        self::associateTable();
        parent::__construct($id, $id_lang, $id_shop);

        $this->id_product_categories = (!empty($this->id_product_categories)
            && !empty(unserialize($this->id_product_categories)))
                ? unserialize($this->id_product_categories)
                : array();
        $this->id_cms_categories = (!empty($this->id_cms_categories)
            && !empty(unserialize($this->id_cms_categories)))
                ? unserialize($this->id_cms_categories)
                : array();
    }

    public function save($null_values = false, $auto_date = true)
    {
        /*$this->serializeIdCategories();*/
        $result = parent::save($null_values, $auto_date);

        // Update group selection
        if (Tools::isSubmit('groupBox')) {
            $this->updateGroup($this->groupBox);
        }

        return $result;
    }

    public function add($auto_date = true, $null_values = false)
    {
        $this->serializeIdCategories();
        $result = parent::add($auto_date, $null_values);

        // Update group selection
        if (Tools::isSubmit('groupBox')) {
            $this->updateGroup($this->groupBox);
        }

        return $result;
    }

    public function update($null_values = false)
    {
        $this->serializeIdCategories();
        $result = parent::update($null_values);

        // Update group selection
        if (Tools::isSubmit('groupBox')) {
            $this->updateGroup($this->groupBox);
        }

        return $result;
    }

    public function serializeIdCategories()
    {
        $this->id_product_categories = (!empty(Tools::getValue('id_product_categories')))
                                            ? serialize(Tools::getValue('id_product_categories'))
                                            : serialize($this->id_product_categories);

        $this->id_cms_categories = (!empty(Tools::getValue('id_cms_categories')))
                                        ? serialize(Tools::getValue('id_cms_categories'))
                                        : serialize($this->id_cms_categories);
    }

    public static function getCurrentIdCopyrightSettings()
    {
        self::associateTable();
        return Db::getInstance()->getValue(
            'SELECT cs.' . self::$definition['primary'] . ' FROM ' . _DB_PREFIX_ . self::$definition['table'] . ' AS cs 
            ' . Shop::addSqlAssociation(self::$definition['table'], 'cs')
        );
    }

    public static function associateTable()
    {
        if (!Shop::getAssoTable(self::$definition['table'])) {
            Shop::addTableAssociation(self::$definition['table'], array('type' => 'shop'));
        }
    }

    public function getGroups()
    {
        $cache_id = 'CopyrightSettings::getGroups_'.(int)$this->id;
        if (!Cache::isStored($cache_id)) {
            $result = Db::getInstance()->executeS('
			SELECT csg.`id_profile`
			FROM '._DB_PREFIX_.self::$definition['table'] . '_registration csg
			WHERE csg.`' . self::$definition['primary'] . '` = '.(int)$this->id);
            $groups = array();
            foreach ($result as $group) {
                $groups[] = $group['id_profile'];
            }
            Cache::store($cache_id, $groups);
            return $groups;
        }
        return Cache::retrieve($cache_id);
    }

    public function updateGroup($list)
    {
        $this->cleanGroups();
        if (empty($list)) {
            $list = array(
                Configuration::get('PS_UNIDENTIFIED_GROUP'),
                Configuration::get('PS_GUEST_GROUP'),
                Configuration::get('PS_CUSTOMER_GROUP')
            );
        }
        $this->addGroups($list);
    }

    public function cleanGroups()
    {
        return Db::getInstance()
            ->delete(self::$definition['table'] . '_registration', self::$definition['primary'] . ' = '.(int)$this->id);
    }

    public function addGroups($groups)
    {
        foreach ($groups as $group) {
            if ($group !== false) {
                $data = array(self::$definition['primary'] => (int)$this->id, 'id_profile' => (int)$group);
                Db::getInstance()
                    ->insert(self::$definition['table'] . '_registration', $data);
            }
        }
    }

    public function accessCategoryProduct($id_category_default)
    {
        if (empty($id_category_default) || !is_numeric($id_category_default)) {
            throw new PrestaShopException('$id_category_default must be a number');
        }

        //if a id category product in id_categories this product is forbidden registration
        return !in_array($id_category_default, $this->id_product_categories);
    }

    public function accessCategoryCms($id_category_default)
    {
        if (empty($id_category_default) || !is_numeric($id_category_default)) {
            throw new PrestaShopException('$id_category_default must be a number');
        }

        //if a id category cms in id_categories this cms is forbidden registration
        return !in_array($id_category_default, $this->id_cms_categories);
    }
    
    public function accessGroups()
    {
        $context = ContextCore::getContext();
        $groups = $this->getGroups();

        return !in_array($context->employee->id_profile, $groups);
    }

    public static function install()
    {
        if (!self::createTables()) {
            return false;
        }

        return true;
    }

    public static function unInstall()
    {
        if (!self::dropTables()) {
            return false;
        }

        return true;
    }

    public static function createTables()
    {
        $tables = array();

        $tables[] = 'CREATE TABLE IF NOT EXISTS ' . _DB_PREFIX_ . self::$definition['table'] . ' (
                        `' . self::$definition['primary'] . '` INT(11) NOT NULL PRIMARY KEY AUTO_INCREMENT,
                        `date_add` TIMESTAMP,
                        `date_upd` TIMESTAMP,
                        `id_content_category` INT(11) NOT NULL,
                        `active` BOOL DEFAULT 0,
                        `id_product_categories` BLOB NOT NULL,
                        `id_cms_categories` BLOB NOT NULL,
                        `available_credits_count` INT(11)
                    ) ENGINE=InnoDB CHARACTER SET=UTF8;';

        $tables[] = 'CREATE TABLE IF NOT EXISTS ' . _DB_PREFIX_ . self::$definition['table'] . '_lang (
                        `' . self::$definition['primary'] . '` INT(11) NOT NULL,
                        `copyright_notice` TEXT NOT NULL,
                        `id_lang` INT(11) NOT NULL,
                        `id_shop` INT(11) NOT NULL
                    ) ENGINE=InnoDB CHARACTER SET=UTF8;';

        $tables[] = 'CREATE TABLE IF NOT EXISTS ' . _DB_PREFIX_ . self::$definition['table'] . '_shop (
                        `' . self::$definition['primary'] . '` INT(11) NOT NULL,
                        `id_product_categories` BLOB NOT NULL,
                        `id_cms_categories` BLOB NOT NULL,
                        `id_shop` INT(11) NOT NULL,
                        PRIMARY KEY (`' . self::$definition['primary'] . '`,`id_shop`),
                        KEY (`id_shop`)
                    ) ENGINE=InnoDB CHARACTER SET=UTF8;';

        $tables[] = 'CREATE TABLE IF NOT EXISTS ' . _DB_PREFIX_ . self::$definition['table'] . '_registration (
                        `' . self::$definition['primary'] . '` INT(11) NOT NULL,
                        `id_profile` INT(11) NOT NULL
                    ) ENGINE=InnoDB CHARACTER SET=UTF8;';

        if (!self::execute($tables)) {
            return false;
        }

        return true;
    }

    public static function dropTables()
    {
        $tables = array();

        $tables[] = 'DROP TABLE IF EXISTS ' . _DB_PREFIX_ . self::$definition['table'] . ';';
        $tables[] = 'DROP TABLE IF EXISTS ' . _DB_PREFIX_ . self::$definition['table'] . '_lang;';
        $tables[] = 'DROP TABLE IF EXISTS ' . _DB_PREFIX_ . self::$definition['table'] . '_shop;';
        $tables[] = 'DROP TABLE IF EXISTS ' . _DB_PREFIX_ . self::$definition['table'] . '_registration;';

        if (!self::execute($tables)) {
            return false;
        }

        return true;
    }

    public static function execute(array $sqls)
    {
        foreach ($sqls as $sql) {
            if (!Db::getInstance()->execute($sql)) {
                return false;
            }
        }

        return true;
    }
}
