<?php
/**
 * 2007-2017 Copyright
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author Objective Concept - Copyright
 *  @copyright Copyright
 *  @package   copyright
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

class CopyrightUser extends ObjectModel
{
    public $cat = '';

    public $promo_code = '';

    public $tel;

    public $email;

    public $password;

    public $first_name;

    public $last_name;

    public $copyright_credits = 0;

    public $copyright_web_credits = 0;

    public $pages_capacity = 0;

    public $id_client = '';

    public $date_add ;

    public $date_upd;

    public $IDF = 135;

    public static $definition = array(
        'table' => 'copyright_user',
        'primary' => 'id_copyright_user',
        'multilang_shop' => true,
        'fields' => array(
            'cat' => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'promo_code' => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'tel' => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'email' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true),
            'first_name' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true),
            'last_name' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true),
            'copyright_credits' => array('type' => self::TYPE_INT, 'validate' => 'isInt'),
            'copyright_web_credits' => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'pages_capacity' => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'id_client' => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'IDF' => array('type' => self::TYPE_STRING, 'validate' => 'isInt'),
            'date_add' => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'date_upd' => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
        ),
    );
    
    public static function getCurrentIdCopyrightUser()
    {
        return Db::getInstance()->getValue(
            'SELECT cu.`' . self::$definition['primary'] . '` 
            FROM ' . _DB_PREFIX_ . self::$definition['table'] . ' AS cu'
        );
    }
    
    public static function getCopyrightUser()
    {
        if ($id_copyright_user = CopyrightUser::getCurrentIdCopyrightUser()) {
            return new self($id_copyright_user);
        }

        return false;
    }

    public function register()
    {
        if (empty($this->email) && empty($this->password)) {
            throw new PrestaShopException("Email and password don't empty, registering a user after saved.");
        }

        $data = array(
            'User' 	=> array(
                'Cat' 					=> null,
                'PromoCode' 			=> $this->promo_code,
                'Tel' 					=> $this->tel,
                'Email'					=> $this->email,
                'Password'				=> $this->password,
                'FirstName' 			=> $this->first_name,
                'LastName'				=> $this->last_name,
                'CopyrightCredits' 		=> 0,
                'CopyrightWebCredits' 	=> 0,
                'PagesCapacity' 		=> 0,
                'ID_client'			    => null
            ),
            'IDF' 	=> $this->IDF,
        );

        $register_result = CopyrightConnect::getInstance()->api_auth('/api/User/register', true, $data);

        if (isset($register_result['ID_client'])) {
            $this->cat = $register_result['Cat'];
            $this->promo_code = $register_result['PromoCode'];
            $this->copyright_credits = $register_result['CopyrightCredits'];
            $this->copyright_web_credits = $register_result['CopyrightWebCredits'];
            $this->pages_capacity = $register_result['PagesCapacity'];
            $this->id_client = $register_result['ID_client'];

            if ($this->activate()
                && $result = $this->auth()) {
                if (isset($result['Message'])) {
                    return $result;
                }
            }

            return $this->save();
        }
    }

    public function auth()
    {
        if (empty($this->email) && empty($this->password)) {
            throw new PrestaShopException("Email and password don't empty, registering a user after saved.");
        }

        $data_for_auth = array(
            "Cat" => null,
            "PromoCode" => null,
            "Tel" => null,
            "Email" => $this->email,
            "Password" => $this->password,
            "FirstName" => null,
            "LastName" => null,
            "CopyrightCredits" => 0,
            "CopyrightWebCredits" => 0,
            "PagesCapacity" => 0,
            "ID_client" => null
        );

        $auth_result = CopyrightConnect::getInstance()->api_auth('/api/User/auth', true, $data_for_auth);

        if (!isset($auth_result['Message'])) {
            $this->cat = $auth_result['Cat'];
            $this->promo_code = $auth_result['PromoCode'];
            $this->tel = $auth_result['Tel'];
            $this->first_name = $auth_result['FirstName'];
            $this->last_name = $auth_result['LastName'];
            $this->copyright_credits = $auth_result['CopyrightCredits'];
            $this->copyright_web_credits = $auth_result['CopyrightWebCredits'];
            $this->pages_capacity = $auth_result['PagesCapacity'];
            $this->id_client = $auth_result['ID_client'];

            return $this->save();
        } else {
            return $auth_result;
        }
    }

    protected function activate()
    {
        if (empty($this->id_client)) {
            throw new PrestaShopException("id_client don't empty, activate a user after register.");
        }

        return CopyrightConnect::getInstance()->api_auth('/api/User/activateaccount?ID_client=' . $this->id_client);
    }

    public function checkCustomer()
    {
        if (empty($this->id_client)) {
            throw new PrestaShopException("id_client doesn't empty, activate a user after register.");
        }

        $copyright_settings = new CopyrightSettings(CopyrightSettings::getCurrentIdCopyrightSettings());

        $checkcustomer = CopyrightConnect::getInstance()
            ->api_auth('/api/User/checkcustomer?ID_client=' . $this->id_client);
        
        if (!empty($checkcustomer['ActiveIDF'])) {
            $this->IDF = $checkcustomer['ActiveIDF'];
            /* Update total number */
            /*$formulas = CopyrightConnect::getInstance()->api_auth('/api/User/formulas?typePlugin=2');
            foreach ($formulas as $formula) {
                if ($checkcustomer['ActiveIDF'] == $formula['IDF']) {
                    $copyright_settings->available_credits_count = $formula['Qty'];
                    break;
                }
            }*/

            /* Update availaible credits */
            /*$credits = CopyrightConnect::getInstance()
                ->api_auth('/api/Copyright/credits?catalog=' . $this->cat . '&ID_client=' . $this->id_client);*/

            $copyright_settings->available_credits_count = $checkcustomer['Credits']['InitialCreditsCopyright'];

            /*if ($credits['Copyright'] == $copyright_settings->available_credits_count) {
                $this->copyright_credits = $credits['Copyright'];
            }*/
            $this->copyright_credits = $checkcustomer['Credits']['Copyright'];
            $this->save();

            return ($copyright_settings->save() && $this->save());
        }
    }

    public static function deleteOneCredit()
    {
        $copyright_user = CopyrightUser::getCopyrightUser();
        $copyright_user->copyright_credits--;

        //Call the service to be managed
        $urlCron = COPYRIGHT_CALLBACK;
        CopyrightConnect::getInstance()
            ->api_auth('/api/Copyright/prepareCopyrightRequest?callback='.$urlCron.'&catalog=' . $copyright_user->cat . '&ID_client=' . $copyright_user->id_client);

        return $copyright_user->save();
    }

    public function getUsedCreditsCount()
    {
        $copyright_settings = new CopyrightSettings(CopyrightSettings::getCurrentIdCopyrightSettings());
        return ($copyright_settings->available_credits_count - $this->copyright_credits);
    }

    public static function install()
    {
        if (!self::createTables()) {
            return false;
        }

        return true;
    }

    public static function unInstall()
    {
        if (!self::dropTables()) {
            return false;
        }

        return true;
    }

    public static function createTables()
    {
        $tables = array();

        $tables[] = 'CREATE TABLE IF NOT EXISTS ' . _DB_PREFIX_ . 'copyright_user (
                        `id_copyright_user` INT(11) NOT NULL PRIMARY KEY AUTO_INCREMENT,
                        `date_add` TIMESTAMP,
                        `date_upd` TIMESTAMP,
                        `cat` CHAR(255),
                        `promo_code` CHAR(255),
                        `tel` CHAR(255),
                        `email` CHAR(255),
                        `password` CHAR(255),
                        `first_name` CHAR(255),
                        `last_name` CHAR(255),
                        `copyright_credits` INT(11) NOT NULL,
                        `copyright_web_credits` INT(11),
                        `pages_capacity` CHAR(255),
                        `id_client` CHAR(255),
                        `IDF` INT(11)
                    ) ENGINE=InnoDB CHARACTER SET=UTF8;';

        if (!self::execute($tables)) {
            return false;
        }

        return true;
    }

    public static function dropTables()
    {
        $tables = array();

        $tables[] = 'DROP TABLE IF EXISTS ' . _DB_PREFIX_ . 'copyright_user;';

        if (!self::execute($tables)) {
            return false;
        }

        return true;
    }

    public static function execute(array $sqls)
    {
        foreach ($sqls as $sql) {
            if (!Db::getInstance()->execute($sql)) {
                return false;
            }
        }

        return true;
    }
}
