<?php
/**
 * 2007-2017 Copyright
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author Objective Concept - Copyright
 * @copyright Copyright
 * @package   copyright
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

class AdminCopyrightSettingsAuthController extends ModuleAdminController
{
    protected $copyright_user;

    public function __construct()
    {
        $this->bootstrap = true;
        $this->table = 'copyright_user';
        $this->className = 'CopyrightUser';
        $this->lang = true;

        parent::__construct();
    }

    public function init()
    {
        parent::init();

        if (CopyrightUser::getCopyrightUser()) {
            $this->redirect_after = $this->context->link->getAdminLink('AdminCopyrightSettings');
        }

        if (Tools::isSubmit('submitAdd' . $this->table)) {
            $this->action = 'auth';
        }
    }

    public function renderList()
    {
        $content = '';

        $content .= $this->renderAuthUserForm();

        $vars = array(
            'link' => $this->context->link,
            'content' => $content,
            'active' => 'auth'
        );

        $content = $this->module->renderTemplate('auth_tabs.smarty', 'admin', $vars);

        return $content;
    }

    public function renderAuthUserForm()
    {
        if (!$this->loadObject(true)) {
            return;
        }

        $this->informations[] = $this->l('Authorization may take several minutes!');

        $this->fields_form = array(
            'tinymce' => true,
            'legend' => array(
                'title' => $this->l('Authorization form'),
                'icon' => 'icon-tags',
                'class' => 'col-md-3'
            ),
            'input' => array(
                array(
                    'type' => 'text',
                    'label' => $this->l('Email.'),
                    'name' => 'email',
                    'required' => true,
                    'col' => 3
                ),
                array(
                    'type' => 'password',
                    'label' => $this->l('Password.'),
                    'name' => 'password',
                    'required' => true,
                ),
            ),
            'submit' => array(
                'title' => $this->l('Sign up'),
                'name' => 'submitAdd' . $this->table,
                'icon' => 'icon-check-circle-o',
            )
        );

        return parent::renderForm();
    }

    public function processAuth()
    {
        $copyright_user = new CopyrightUser();
        $copyright_user->email = Tools::getValue('email');
        $copyright_user->password = Tools::getValue('password');

        if ($result = $copyright_user->auth()) {
            if (isset($result['Message'])) {
                $this->errors[] = $result['Message'];
                $copyright_user->delete();
            } else {

                $copyright_user->checkCustomer();

                $this->redirect_after = $this->context->link->getAdminLink('AdminCopyrightSettings');
            }
        }
    }

    public function ajaxProcessAuth()
    {
        $response = array();

        $copyright_user = new CopyrightUser();
        $copyright_user->email = Tools::getValue('email');
        $copyright_user->password = Tools::getValue('password');

        if (!isset($response['copyright_error'])
            && empty($response['copyright_error'])
            && $result = $copyright_user->auth()
        ) {
            if (isset($result['Message'])) {
                /*$response['copyright_error'] = $result['Message'];*/
                $response['copyright_error'] = 'Wrong Login or Password!';
                $copyright_user->delete();
            } else {

                $copyright_user->checkCustomer();

                $response['redirect_after'] = $this->context->link->getAdminLink('AdminCopyrightSettings');
            }
        } else {
            $copyright_user->delete();
        }

        die(Tools::jsonEncode($response));
    }
}
