<?php
/**
 * 2007-2017 Copyright
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author Objective Concept - Copyright
 * @copyright Copyright
 * @package   copyright
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

class AdminCopyrightSettingsController extends AdminController
{
    protected $copyright_user;

    public function __construct()
    {
        $this->bootstrap = true;
        $this->table = 'copyright_settings';
        $this->className = 'CopyrightSettings';
        $this->lang = true;

        parent::__construct();

    }

    public function init()
    {
        parent::init();

        if (!CopyrightUser::getCopyrightUser()) {
            $this->redirect_after = $this->context->link->getAdminLink('AdminCopyrightSettingsAuth');
        }
        else {
            if (!CopyrightUser::getCopyrightUser()) {
                $this->redirect_after = $this->context->link->getAdminLink('AdminCopyrightSettingsAuth');
            }
            //create a new user or get a old copyright user
            $this->copyright_user = CopyrightUser::getCopyrightUser();
            $this->copyright_user->checkCustomer();
        }
    }

    public function setMedia()
    {
        parent::setMedia();

        $this->addCSS(array(
            COPYRIGHT_DIR . 'views/css/style.css',
        ));
    }

    public function renderList()
    {
        $content = '';

        //create a new user or get a old copyright user
        $copyright_user = CopyrightUser::getCopyrightUser();

        $copyright = Module::getInstanceByName('copyright');

        //render bags for shopping
        $formulas = CopyrightConnect::getInstance()->api_auth('/api/User/formulas?typePlugin=2');
        $vars = array(
            'formulas' => $formulas,
            'view_dir' => $copyright->getViewUri(),
            'copyright_user' => $copyright_user,
            'admin_dir' => Tools::getHttpHost(true) . __PS_BASE_URI__
                . basename(_PS_ADMIN_DIR_)
                . '/' . $this->context->link->getAdminLink('AdminCopyrightSettings'),
        );
        $content .= $copyright->renderTemplate('formulas.tpl', 'admin', $vars);

        //render category products
        $content .= $this->renderSettingsForm();

        $vars = array(
            'link' => $this->context->link,
            'content' => $content,
            'active' => 'settings',
        );

        $content = $copyright->renderTemplate('settings_tabs.smarty', 'admin', $vars);

        if (!empty($copyright_user) && !$copyright_user->copyright_credits) {
            $message = $copyright->renderWarning($this->l(
                'You do not have sufficient credits to protect this content. '
                . 'Choose a more advanced plan to purchase more credits.'
            ));

            $content = $message . $content;
        }

        return $content;
    }

    protected function loadObject($opt = false)
    {
        if (!isset($this->className) || empty($this->className)) {
            return true;
        }

        $id = (int)CopyrightSettings::getCurrentIdCopyrightSettings();
        if ($id && Validate::isUnsignedId($id)) {
            if (!$this->object) {
                $this->object = new $this->className($id);
            }
            if (Validate::isLoadedObject($this->object)) {
                return $this->object;
            }
            // throw exception
            $this->errors[] = Tools::displayError('The object cannot be loaded (or found)');
            return false;
        } elseif ($opt) {
            if (!$this->object) {
                $this->object = new $this->className();
            }
            return $this->object;
        } else {
            $this->errors[] = Tools::displayError('The object cannot be loaded (the dentifier is missing or invalid)');
            return false;
        }
    }

    public function renderSettingsForm()
    {
        if (!($obj = $this->loadObject(true))) {
            return;
        }

        //add Group access field
        $ps = Profile::getProfiles($this->context->language->id);
        $profiles = array();
        foreach ($ps as $profile) {
            $profiles[] = array(
                'id_group' => $profile['id_profile'],
                'name' => $profile['name'],
            );
        }

        $this->fields_form = array(
            'tinymce' => true,
            'legend' => array(
                'title' => $this->l('Settings'),
                'icon' => 'icon-tags'
            ),
            'input' => array(
                array(
                    'type' => 'categories',
                    'label' => $this->l('Deny Product Categories'),
                    'name' => 'id_product_categories',
                    'id' => 'product-categories-tree',
                    'tree' => array(
                        'id' => 'product-categories-tree',
                        'selected_categories' => $obj->id_product_categories,
                        'root_category' => $this->context->shop->getCategory(),
                        'use_checkbox' => true,
                        'use_search' => true,
                    )
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Deny Cms Categories'),
                    'name' => 'id_cms_categories[]',
                    'multiple' => true,
                    'options' => array(
                        'query' => CMSCategory::getSimpleCategories($this->context->language->id),
                        'id' => 'id_cms_category',
                        'name' => 'name',
                        'selected' => 'selected',
                    ),
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Content category'),
                    'name' => 'id_content_category',
                    'options' => array(
                        'query' => CopyrightConnect::getInstance()
                            ->api_auth('/api/Copyright/sectors?catalog=' . $this->copyright_user->cat),
                        'id' => 'idField',
                        'name' => 'valueField',
                    )
                ),
                array(
                    'type' => 'textarea',
                    'label' => $this->l('Copyright notice'),
                    'name' => 'copyright_notice',
                    'autoload_rte' => true,
                    'lang' => true,
                ),
                array(
                    'type' => 'group',
                    'label' => $this->l('Deny access to copyright registration'),
                    'name' => 'groupBox',
                    'values' => $profiles,
                    'hint' => $this->l(
                        'Mark all of the admin groups which you would '
                        . 'like to have access to copyright registration.'
                    )
                )
            ),
            'submit' => array(
                'title' => $this->l('Save'),
                'name' => 'submitAdd' . $this->table,
            )
        );


        foreach (Language::getLanguages(true) as $lang) {
            if (empty($obj->copyright_notice[$lang['id_lang']])) {
                $this->fields_value['copyright_notice'][$lang['id_lang']] = 'Notice: This article has been protected by 
                Copyright. See {certificate="copyright certificate"}';
            }
        }

        // Display this field only if multistore option is enabled AND there are several stores configured
        if (Shop::isFeatureActive()) {
            $this->fields_form['input'][] = array(
                'type' => 'shop',
                'label' => $this->l('Shop association'),
                'name' => 'checkBoxShopAsso',
            );
        }

        // Added values of object Group
        $category_groups_ids = $obj->getGroups();
        foreach ($profiles as $group) {
            $default = in_array($group['id_group'], $category_groups_ids);
            $value = Tools::getValue('groupBox_' . $group['id_group'], $default);
            $this->fields_value['groupBox_' . $group['id_group']] = $value;
        }

        foreach ($obj->id_cms_categories as $id_cms_category) {
            $this->fields_value['id_cms_categories[]'][] = $id_cms_category;
        }

        return parent::renderForm();
    }
}
