<?php
/**
 * 2007-2017 Copyright
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author Objective Concept - Copyright
 * @copyright Copyright
 * @package   copyright
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

class AdminCopyrightSettingsHistoryController extends ModuleAdminController
{
    public function __construct()
    {
        $this->bootstrap = true;
        $this->explicitSelect = true;
        $this->bulk_actions = array(
            'delete' => array(
                'text' => $this->l('Delete selected'),
                'icon' => 'icon-trash',
                'confirm' => $this->l('Delete selected items?')
            )
        );

        $this->context = Context::getContext();

        parent::__construct();

    }

    public function initPageHeaderToolbar()
    {
        $this->page_header_toolbar_btn['refresh'] = array(
            'href' => $this->context->link->getAdminLink('AdminCopyrightSettingsHistory').'&update',
            'desc' => $this->l('Update Certificates', null, null, false),
            'icon' => 'process-icon-refresh'
        );

        parent::initPageHeaderToolbar();
    }

    public function renderList()
    {
        $content = '';

        $content .= $this->renderProductCopyrightList();
        $content .= $this->renderCmsCopyrightList();
        $content .= $this->renderCategoryCopyrightList();

        $vars = array(
            'link' => $this->context->link,
            'content' => $content,
            'active' => 'history'
        );

        $content = $this->module->renderTemplate('settings_tabs.smarty', 'admin', $vars);

        $copyright_user = CopyrightUser::getCopyrightUser();

        if (!empty($copyright_user) && !$copyright_user->copyright_credits) {
            $message = $this->module->renderWarning(
                $this->l(
                    'You do not have sufficient credits to protect this content. 
                    Choose a more advanced plan to purchase more credits.'
                ) .
                '<a href="' . $this->context->link->getAdminLink('AdminCopyrightSettings') . '">'
                . $this->l('Click here to upgrade your account') .
                '</a>'
            );

            $content = $message . $content;
        }

        return $content;
    }

    public function renderProductCopyrightList()
    {
        $content = CopyrightProduct::getAllProducts();

        $fields_list = array(
            'id_copyright_product' => array(
                'title' => $this->l('ID'),
                'align' => 'center',
                'class' => 'fixed-width-xs',

            ),
            'id_product' => array(
                'title' => $this->l('Name'),
                'type' => 'text',
                'callback' => 'getProductName',
                'search' => false,
                'align' => 'center',
                'class' => 'col-xs-3',
            ),
            'date_add' => array(
                'title' => $this->l('Date added'),
                'type' => 'date',
                'align' => 'center',
                'class' => 'col-xs-3',
            ),
            'certificate_url' => array(
                'title' => $this->l('Certificate'),
                'type' => 'text',
                'search' => false,
                'callback' => 'getCertificateLink',
                'align' => 'center',
                'class' => 'col-xs-3',
            ),
            'registered_file_url' => array(
                'title' => $this->l('Registered file'),
                'type' => 'text',
                'search' => false,
                'callback' => 'getRegisteredFileLink',
                'align' => 'center',
                'class' => 'col-xs-3',
            ),
        );

        $helper = new HelperList();
        $helper->shopLinkType = '';
        $helper->actions = array("");
        $helper->module = $this->module;
        $helper->listTotal = count($content);
        $helper->identifier = CopyrightProduct::$definition['primary'];
        $helper->title = $this->l('Products list');

        $helper->table = CopyrightProduct::$definition['table'];
        $helper->token = Tools::getAdminTokenLite('AdminCopyrightSettingsHistory');
        $helper->currentIndex = self::$currentIndex . '&configure=' . $this->module->name;

        return $helper->generateList($content, $fields_list);
    }

    public function renderCmsCopyrightList()
    {
        $content = CopyrightCms::getAllCms();

        $fields_list = array(
            'id_copyright_cms' => array(
                'title' => $this->l('ID'),
                'align' => 'center',
                'class' => 'fixed-width-xs',
            ),
            'id_cms' => array(
                'title' => $this->l('Name'),
                'type' => 'text',
                'callback' => 'getCmsName',
                'search' => false,
                'align' => 'center',
                'class' => 'col-xs-3',
            ),
            'date_add' => array(
                'title' => $this->l('Date added'),
                'type' => 'date',
                'align' => 'center',
                'class' => 'col-xs-3',
            ),
            'certificate_url' => array(
                'title' => $this->l('Certificate'),
                'type' => 'text',
                'search' => false,
                'callback' => 'getCertificateLink',
                'align' => 'center',
                'class' => 'col-xs-3',
            ),
            'registered_file_url' => array(
                'title' => $this->l('Registered file'),
                'type' => 'text',
                'search' => false,
                'callback' => 'getRegisteredFileLink',
                'align' => 'center',
                'class' => 'col-xs-3',
            ),
        );

        $helper = new HelperList();
        $helper->shopLinkType = '';
        $helper->actions = array("");
        $helper->module = $this->module;
        $helper->listTotal = count($content);
        $helper->identifier = CopyrightCms::$definition['primary'];
        $helper->title = $this->l('Cms list');
        $helper->table = CopyrightCms::$definition['table'];
        $helper->token = Tools::getAdminTokenLite('AdminCopyrightSettingsHistory');
        $helper->currentIndex = self::$currentIndex . '&configure=' . $this->module->name;

        return $helper->generateList($content, $fields_list);
    }

    public function renderCategoryCopyrightList()
    {
        $content = CopyrightCategory::getAllCategories();

        $fields_list = array(
            'id_copyright_category' => array(
                'title' => $this->l('ID'),
                'align' => 'center',
                'class' => 'fixed-width-xs',
            ),
            'id_category' => array(
                'title' => $this->l('Name'),
                'type' => 'text',
                'callback' => 'getCategoryName',
                'search' => false,
                'align' => 'center',
                'class' => 'col-xs-3',
            ),
            'date_add' => array(
                'title' => $this->l('Date added'),
                'type' => 'date',
                'align' => 'center',
                'class' => 'col-xs-3',
            ),
            'certificate_url' => array(
                'title' => $this->l('Certificate'),
                'type' => 'text',
                'search' => false,
                'callback' => 'getCertificateLink',
                'align' => 'center',
                'class' => 'col-xs-3',
            ),
            'registered_file_url' => array(
                'title' => $this->l('Registered file'),
                'type' => 'text',
                'search' => false,
                'callback' => 'getRegisteredFileLink',
                'align' => 'center',
                'class' => 'col-xs-3',
            ),
        );

        $helper = new HelperList();
        $helper->shopLinkType = '';
        $helper->actions = array("");
        $helper->module = $this->module;
        $helper->listTotal = count($content);
        $helper->identifier = CopyrightCategory::$definition['primary'];
        $helper->title = $this->l('Category list');
        $helper->table = CopyrightCategory::$definition['table'];
        $helper->token = Tools::getAdminTokenLite('AdminCopyrightSettingsHistory');
        $helper->currentIndex = self::$currentIndex . '&configure=' . $this->module->name;

        return $helper->generateList($content, $fields_list);
    }

    public static function getProductName($id_product)
    {
        $context = Context::getContext();
        $product = new Product($id_product, true, $context->language->id);

        return $product->name;
    }

    public static function getCmsName($id_cms)
    {
        $context = Context::getContext();
        $cms = new CMS($id_cms, $context->language->id);

        return $cms->meta_title;
    }

    public static function getCategoryName($id_category)
    {
        $context = Context::getContext();
        $category = new Category($id_category, $context->language->id);

        return $category->name;
    }

    public static function getCertificateLink($link)
    {
        $copyright = Module::getInstanceByName('copyright');

        $vars = array(
            'href' => $link ? $link : false,
            'view_dir' => $copyright->getViewUri(),
        );

        return $copyright->renderTemplate('certificate_link.tpl', 'admin', $vars);
    }

    public static function getRegisteredFileLink($link)
    {
        $copyright = Module::getInstanceByName('copyright');

        $vars = array(
            'href' => $link ? $link : false,
            'view_dir' => $copyright->getViewUri(),
        );

        return $copyright->renderTemplate('registered_file_link.tpl', 'admin', $vars);
    }

    public function initProcess()
    {

        if (isset($_GET['update'])) {
            try {

                $this->module->processAll();

            } catch (Exception $e) {
                $this->errors[] = $e->getMessage();
            }

            Tools::redirectAdmin(self::$currentIndex.'&conf=4'.'&token='.$this->token);
        }
    }
}
