<?php
/**
 * 2007-2017 Copyright
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author Objective Concept - Copyright
 * @copyright Copyright
 * @package   copyright
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

class AdminCopyrightSettingsRegisterController extends ModuleAdminController
{
    protected $copyright_user;

    public function __construct()
    {
        $this->bootstrap = true;
        $this->table = 'copyright_user';
        $this->className = 'CopyrightUser';
        $this->lang = true;

        parent::__construct();
    }

    public function init()
    {
        parent::init();

        if (CopyrightUser::getCopyrightUser()) {
            $this->redirect_after = $this->context->link->getAdminLink('AdminCopyrightSettings');
        }

        if (Tools::isSubmit('submitAdd' . $this->table)) {
            $this->action = 'register';
        }
    }

    public function renderList()
    {
        $content = '';

        $content .= $this->renderRegisterUserForm();

        $vars = array(
            'link' => $this->context->link,
            'content' => $content,
            'active' => 'register'
        );

        $content = $this->module->renderTemplate('auth_tabs.smarty', 'admin', $vars);

        return $content;
    }

    public function renderRegisterUserForm()
    {
        if (!$this->loadObject(true)) {
            return;
        }

        $this->informations[] = $this->l('Register for free and get 3 copyrights free on to your account!');
        $this->informations[] = $this->l('Registration may take several minutes!');

        $this->fields_form = array(
            'tinymce' => true,
            'legend' => array(
                'title' => $this->l('Registration form'),
                'icon' => 'icon-tags'
            ),
            'input' => array(
                array(
                    'type' => 'text',
                    'label' => $this->l('Email.'),
                    'name' => 'email',
                    'required' => true,
                    'col' => 3,
                ),
                array(
                    'type' => 'password',
                    'label' => $this->l('Password.'),
                    'name' => 'password',
                    'required' => true,
                    'col' => 3,
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('First Name.'),
                    'name' => 'first_name',
                    'required' => true,
                    'col' => 3,
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Last Name.'),
                    'name' => 'last_name',
                    'required' => true,
                    'col' => 3
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Phone.'),
                    'name' => 'tel',
                    'col' => 3,
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Promo Code'),
                    'name' => 'promo_code',
                    'col' => 3,
                ),
            ),
            'submit' => array(
                'title' => $this->l('Sign Up'),
                'name' => 'submitAdd' . $this->table,
                'icon' => 'icon-check-circle-o'
            )
        );

        return parent::renderForm();
    }

    public function processRegister()
    {

        $copyright_user = new CopyrightUser();
        $copyright_user->email = Tools::getValue('email');
        $copyright_user->password = Tools::getValue('password');
        $copyright_user->first_name = Tools::getValue('first_name');
        $copyright_user->last_name = Tools::getValue('last_name');
        $copyright_user->tel = Tools::getValue('tel');
        $copyright_user->promo_code = Tools::getValue('promo_code');
        $copyright_user->save();

        if ($copyright_user->first_name == $copyright_user->last_name) {
            $this->errors[] = $this->l('First Name and Last Name may not be the same.');
        }

        if (empty($this->errors)
            && $result = $copyright_user->register()
        ) {
            if (isset($result['Message'])) {
                $this->errors[] = $result['Message'];
                $copyright_user->delete();

            } else {

                $copyright_user->checkCustomer();

                $this->redirect_after = $this->context->link->getAdminLink('AdminCopyrightSettings');
            }
        } else {
            $copyright_user->delete();
        }

    }

    public function ajaxProcessRegister()
    {

        $response = array();

        $copyright_user = new CopyrightUser();
        $copyright_user->email = Tools::getValue('email');
        $copyright_user->password = Tools::getValue('password');
        $copyright_user->first_name = Tools::getValue('first_name');
        $copyright_user->last_name = Tools::getValue('last_name');
        $copyright_user->tel = Tools::getValue('tel');
        $copyright_user->promo_code = Tools::getValue('promo_code');

        try {
            $copyright_user->save();
        } catch (Exception $e) {
            $response['copyright_error'] = $e->getMessage();
        }

        if (empty($copyright_user->email)) {
            $this->errors[] = $this->l('Enter your email.');
        }
        if (empty($copyright_user->password)) {
            $this->errors[] = $this->l('Enter your password.');
        }
        if (empty($copyright_user->first_name)) {
            $this->errors[] = $this->l('Enter your firstname.');
        }
        if (empty($copyright_user->last_name)) {
            $this->errors[] = $this->l('Enter your lastname.');
        }

        if (!empty($this->errors)) {
            $response['copyright_error'] = '';
            foreach ($this->errors as $error) {
                $response['copyright_error'] .= '<div style="padding: 0 30px 0 30px;">' . $error . '</div>';
            }
        }

        if (!isset($response['copyright_error']) && empty($response['copyright_error'])
            && $result = $copyright_user->register()
        ) {
            if (isset($result['Message'])) {
                $response['copyright_error'] = $result['Message'];
                $copyright_user->delete();

            } else {

                $copyright_user->checkCustomer();

                $response['redirect_after'] = $this->context->link->getAdminLink('AdminCopyrightSettings');
            }
        } else {
            $copyright_user->delete();
        }

        die(Tools::jsonEncode($response));
    }
}
