<?php
/**
    * 2007-2017 Copyright
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author Objective Concept - Copyright
*  @copyright Copyright
*  @package   copyright
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if(!defined('_PS_VERSION_'))
    exit;

if(!defined('COPYRIGHT_API_HOST'))
    define('COPYRIGHT_API_HOST', 'https://copyrightapi.com');
if(!defined('COPYRIGHT_FILE_HOST'))
    define('COPYRIGHT_FILE_HOST', 'https://copyrightweb.com');
if(!defined('COPYRIGHT_DIR'))
    define('COPYRIGHT_DIR', _PS_MODULE_DIR_ . 'copyright/');
if(!defined('COPYRIGHT_CALLBACK'))
    define('COPYRIGHT_CALLBACK',_PS_BASE_URL_.'/module/copyright/cron');

require_once COPYRIGHT_DIR . 'CopyrightAutoload.php';
CopyrightAutoload::register();

class Copyright extends Module
{

    public $copyright_errors = array();

    public $hooks = array(

        'displayAdminProductsExtra',
        'actionObjectProductAddAfter',
        'actionObjectProductUpdateAfter',
        'actionAdminProductsListingFieldsModifier',

        'actionObjectCmsAddAfter',
        'actionObjectCmsUpdateAfter',
        'actionAdminCmsListingFieldsModifier',

        'actionObjectCategoryAddAfter',
        'actionObjectCategoryUpdateAfter',
        'actionAdminCategoriesListingFieldsModifier',

        'extraRight',
        'displayBackOfficeHeader',
        'displayBackOfficeTop',
        'actionAdminCmsFormModifier',
        'actionAdminCategoriesFormModifier',
        'header'

    );

    public function __construct()
    {
        $this->name = 'copyright';
        $this->tab = 'front_office_features';
        $this->version = '1.0.0';
        $this->author = 'Copyright';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
        $this->bootstrap = true;
        $this->module_key = 'abde98118b544c96d6ac3aaef4e98a6d';

        parent::__construct();

        $this->displayName = $this->l('Copyright');
        $this->description = $this->l(
            'Copyright Module allows to every owner of PrestaShop website to protect his content.'
        );

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

        if (!Configuration::get('Copyright')) {
            $this->warning = $this->l('No name provided');
        }
    }

    public function install()
    {

        $tab = new Tab();
        $tab->active = 1;
        $tab->class_name = 'AdminCopyrightSettingsHistory';
        $tab->name = array();

        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = $this->l('Copyright');
        }

        $tab->id_parent = 0;
        $tab->position = 0;
        $tab->module = $this->name;
        
        if (!$this->modelsInstall()
            || !$tab->add()
            || !parent::install()
            || !$this->registerHook($this->hooks)
            || !$this->installModuleTab('AdminCopyrightSettingsHistory', $this->l('Copyright History'), $tab->id)
            || !$this->installModuleTab('AdminCopyrightSettings', $this->l('Settings'), $tab->id)
            || !$this->installModuleTab('AdminCopyrightSettingsUpdatePlan', $this->l('Upgrade my plans'), $tab->id)
            || !$this->installModuleTab('AdminCopyrightSettingsAuth', $this->l('Authorization'), $tab->id, 0, false)
            || !$this->installModuleTab('AdminCopyrightSettingsRegister', $this->l('Registration'), $tab->id, 0, false)
            || !$this->installModuleTab('AdminCopyrightSettingsAbout', $this->l('About'), $tab->id)) {
            return false;
        }

        return true;
    }

    public function uninstall()
    {
        if (!$this->modelsUninstall()
            || !$this->uninstallModuleTabs()
            || !parent::uninstall()) {
            return false;
        }

        return true;
    }

    protected function modelsInstall()
    {
        try {

            if (!CopyrightSettings::install()
                || !CopyrightUser::install()
                || !CopyrightProduct::install()
                || !CopyrightCategory::install()
                || !CopyrightCms::install()
                || !CopyrightFile::install()) {
                return false;
            }

        } catch (Exception $e) {
            $this->_errors[] = $e->getMessage();
            return false;
        }

        return true;
    }

    protected function modelsUninstall()
    {
        try {

            if (!CopyrightSettings::unInstall()
                || !CopyrightUser::unInstall()
                || !CopyrightProduct::unInstall()
                || !CopyrightCategory::unInstall()
                || !CopyrightCms::unInstall()
                || !CopyrightFile::unInstall()) {
                return false;
            }

        } catch (Exception $e) {
            $this->_errors[] = $e->getMessage();
            return false;
        }

        return true;
    }

    public function validate($type = '')
    {
        $errors = array();
        $copyright_settings = new CopyrightSettings(CopyrightSettings::getCurrentIdCopyrightSettings());

        if (!$copyright_settings->accessGroups()) {
            $errors[] = Tools::displayError($this->l('You are not allowed to work with the module Copyright.'));
        }

        if (!$copyright_user = CopyrightUser::getCopyrightUser()) {
            $errors[] = Tools::displayError($this->l('You need to create an account.'));
        }

        if (!$copyright_user->copyright_credits) {
            $errors[] = Tools::displayError($this->l('Please purchase more credits by upgrading your plans here.'))
                . ' <a href="' . $this->context->link->getAdminLink('AdminCopyrightSettings') . '">'
                . $this->l('Click here to upgrade your account') .
                '</a>';
        }


        switch ($type) {
            case 'product':

                $product = new Product(Tools::getValue('id_product', 0));
                if (!$copyright_settings->accessCategoryProduct($product->id_category_default)) {
                    $errors[] = Tools::displayError($this->l('Product registration is forbidden from this category.'));
                }

                break;
            case 'cms':
                $cms = new CMS(Tools::getValue('id_cms'));
                if (!$copyright_settings->accessCategoryCms($cms->id_cms_category)) {
                    $errors[] = Tools::displayError($this->l('Cms page registration is forbidden from this category.'));
                }

                break;
        }

        return $errors;
    }

    public function ajaxValidate($type = '')
    {
        $errors = array();
        $copyright_settings = new CopyrightSettings(CopyrightSettings::getCurrentIdCopyrightSettings());

        if (!$copyright_settings->accessGroups()) {
            die(Tools::jsonEncode(
                array(
                    'success' => false,
                    'error' => true,
                    'text' => $this->l('You are not allowed to work with the module Copyright.')
                )
            ));
        }

        if (!$copyright_user = CopyrightUser::getCopyrightUser()) {
            die(Tools::jsonEncode(
                array(
                    'success' => false,
                    'error' => true,
                    'text' => $this->l('You need to create an account.')
                )
            ));
        }

        if (!$copyright_user->copyright_credits) {
            die(Tools::jsonEncode(
                array(
                    'success' => false,
                    'error' => true,
                    'text' => $this->l('Please purchase more credits by upgrading your plans here.')
                        . ' <a href="' . $this->context->link->getAdminLink('AdminCopyrightSettings') . '">'
                        . $this->l('Click here to upgrade your account') .
                        '</a>'
                )
            ));
        }

        switch ($type) {
            case 'product':

                $product = new Product(Tools::getValue('id_product', 0));
                if (!$copyright_settings->accessCategoryProduct($product->id_category_default)) {
                    die(Tools::jsonEncode(
                        array(
                            'success' => false,
                            'error' => true,
                            'text' => $this->l('Product registration is forbidden from this category.')
                        )
                    ));
                }

                break;
            case 'cms':

                $cms = new CMS(Tools::getValue('id_cms'));
                if (!$copyright_settings->accessCategoryCms($cms->id_cms_category)) {
                    die(Tools::jsonEncode(
                        array(
                            'success' => false,
                            'error' => true,
                            'text' => $this->l('Cms page registration is forbidden from this category.')
                        )
                    ));
                }

                break;
        }

        return $errors;
    }

    public function hookDisplayAdminProductsExtra($params)
    {
        $copyright_settings = new CopyrightSettings(CopyrightSettings::getCurrentIdCopyrightSettings());
        $product = new Product(Tools::getValue('id_product', 0));

        //validate access to create a copyright
        $errors = $this->validate('product');
        
        $this->context->smarty->assign(array(
            'view_dir' => $this->getViewUri(),
            'copyright_registration' => CopyrightProduct::getProduct($product->id),
            'copyright_settings' => $copyright_settings,
            'copyright_user' => CopyrightUser::getCopyrightUser(),
            'errors' => $errors,
            'product_in_progress' => CopyrightProduct::inProgress((int)$product->id),
            'register_product' => CopyrightProduct::getRegisterProduct((int)$product->id),
        ));

        return $this->display(__FILE__, 'displayAdminProductsExtra.smarty');
    }
    
    public function hookActionObjectProductAddAfter($params)
    {
        $this->copyrightProductRegistration($params);
    }

    public function hookActionObjectProductUpdateAfter($params)
    {
        $this->copyrightProductRegistration($params);
    }

    public function copyrightProductRegistration($params)
    {
        static $worked = false;

        $product = $params['object'];
        if (!$worked
            &&Tools::isSubmit('copyright_registration')
            && Tools::getValue('copyright_registration')
            && !CopyrightProduct::getRegisterProduct($product->id)) {

            if ($copyright_product = CopyrightProduct::getProduct((int)$product->id)) {
                $copyright_file = new CopyrightFile($copyright_product['id_copyright_file']);
                $copyright_file->is_recurring = 1;
                $copyright_file->save();
            } else {

                $copyright_file = new CopyrightFile();
                $copyright_file->status = 0;
                $copyright_file->id_deposit = 0;
                $copyright_file->is_recurring = 1;
                $copyright_file->save();

                $copyright_product = new CopyrightProduct();
                $copyright_product->id_product = $product->id;
                $copyright_product->id_copyright_file = $copyright_file->id;

                //if success save copyright file then copyright product save
                $copyright_product->save();
            }

            CopyrightUser::deleteOneCredit();

            //if object saving other a module
            $worked = true;

        } else {
            //don't unregistered
        }
    }

    public function hookActionAdminProductsListingFieldsModifier($params)
    {
        $params['fields']['copyright'] = array(
            'title' => $this->l('Copyright'),
            'type' => 'bool',
            'active' => 'copyright',
            'align' => 'text-right',
            'havingFilter' => true,
            'orderby' => false,
            'search' => true,
            'ajax' => true,
        );

        if (isset($params['select'])) {
            CopyrightProduct::associateTable();
            CopyrightFile::associateTable();

            $params['select'] .= ', IF((SELECT 1 FROM '
                . _DB_PREFIX_ . CopyrightProduct::$definition['table']
                . ' AS c_p '
                . Shop::addSqlAssociation(CopyrightProduct::$definition['table'], 'c_p')
                . ' LEFT JOIN ' . _DB_PREFIX_ . CopyrightFile::$definition['table']
                . ' AS cf ON (cf.`id_copyright_file` = c_p.`id_copyright_file`) '
                . Shop::addSqlAssociation(CopyrightFile::$definition['table'], 'cf')
                . ' WHERE c_p.`id_product` = a.`id_product` LIMIT 1), 1, 0) AS `copyright`';
        }
    }

    public function hookActionObjectCmsAddAfter($params)
    {
        $this->copyrightCmsRegistration($params);
    }

    public function hookActionObjectCmsUpdateAfter($params)
    {
        $this->copyrightCmsRegistration($params);
    }

    public function copyrightCmsRegistration($params)
    {
        static $worked = false;

        $cms = $params['object'];
        if (!$worked
            && Tools::isSubmit('copyright_registration')
            && Tools::getValue('copyright_registration')
            && !CopyrightCms::getRegisterCms($cms->id)) {
            if ($copyright_cms = CopyrightCms::getCms((int)$cms->id)) {
                $copyright_file = new CopyrightFile($copyright_cms['id_copyright_file']);
                $copyright_file->is_recurring = 1;
                $copyright_file->save();
            } else {

                $copyright_file = new CopyrightFile();
                $copyright_file->status = 0;
                $copyright_file->id_deposit = 0;
                $copyright_file->is_recurring = 1;
                $copyright_file->save();

                $copyright_cms = new CopyrightCms();
                $copyright_cms->id_cms = $cms->id;
                $copyright_cms->id_copyright_file = $copyright_file->id;

                //if success save copyright file then copyright product save
                $copyright_cms->save();
            }

            CopyrightUser::deleteOneCredit();

            //if object saving other a module
            $worked = true;

        } else {
            //don't unregistered
        }
    }

    public function hookActionAdminCmsListingFieldsModifier($params)
    {
        $params['fields']['copyright'] = array(
            'title' => $this->l('Copyright'),
            'type' => 'bool',
            'active' => 'copyright',
            'align' => 'text-right',
            'havingFilter' => true,
            'orderby' => false,
            'search' => true,
            'ajax' => true,
        );

        if (isset($params['select'])) {
            CopyrightCms::associateTable();
            CopyrightFile::associateTable();

            $params['select'] .= ', IF((SELECT 1 FROM ' . _DB_PREFIX_ . CopyrightCms::$definition['table'] . ' AS cms
                                        ' . Shop::addSqlAssociation(CopyrightCms::$definition['table'], 'cms') . '
                                         LEFT JOIN ' . _DB_PREFIX_ . CopyrightFile::$definition['table'] . ' AS cf 
                                            ON (cf.`id_copyright_file` = cms.`id_copyright_file`)
                                        ' . Shop::addSqlAssociation(CopyrightFile::$definition['table'], 'cf') . ' 
                                         WHERE cms.`id_cms` = a.`id_cms` LIMIT 1), 1, 0) AS `copyright`';
        }
    }

    public function hookActionObjectCategoryAddAfter($params)
    {
        $this->copyrightCategoryRegistration($params);
    }

    public function hookActionObjectCategoryUpdateAfter($params)
    {
        $this->copyrightCategoryRegistration($params);
    }

    public function copyrightCategoryRegistration($params)
    {
        static $worked = false;

        $category = $params['object'];
        if (!$worked
            && Tools::isSubmit('copyright_registration')
            && Tools::getValue('copyright_registration')
            && !CopyrightCategory::getRegisterCategory($category->id)) {
            if ($copyright_category = CopyrightCategory::getCategory((int)$category->id)) {
                $copyright_file = new CopyrightFile($copyright_category['id_copyright_file']);
                $copyright_file->is_recurring = 1;
                $copyright_file->save();
            } else {

                $copyright_file = new CopyrightFile();
                $copyright_file->status = 0;
                $copyright_file->id_deposit = 0;
                $copyright_file->is_recurring = 1;
                $copyright_file->save();

                $copyright_category = new CopyrightCategory();
                $copyright_category->id_category = $category->id;
                $copyright_category->id_copyright_file = $copyright_file->id;

                //if success save copyright file then copyright product save
                $copyright_category->save();
            }

            CopyrightUser::deleteOneCredit();

            //if object saving other a module
            $worked = true;

        } else {
            //don't unregistered
        }
    }

    public function hookActionAdminCategoriesListingFieldsModifier($params)
    {
        $params['fields']['copyright'] = array(
            'title' => $this->l('Copyright'),
            'type' => 'bool',
            'active' => 'copyright',
            'align' => 'text-right',
            'havingFilter' => true,
            'orderby' => false,
            'search' => true,
            'ajax' => true,
        );

        if (isset($params['select'])) {
            CopyrightCategory::associateTable();
            CopyrightFile::associateTable();

            $params['select'] .= ', IF((SELECT 1 FROM ' . _DB_PREFIX_ . CopyrightCategory::$definition['table'] . ' AS c
                                        ' . Shop::addSqlAssociation(CopyrightCategory::$definition['table'], 'c') . '
                                         LEFT JOIN ' . _DB_PREFIX_ . CopyrightFile::$definition['table'] . ' AS cf 
                                            ON (cf.`id_copyright_file` = c.`id_copyright_file`)
                                        ' . Shop::addSqlAssociation(CopyrightFile::$definition['table'], 'cf') . ' 
                                         WHERE c.`id_category` = a.`id_category` LIMIT 1), 1, 0) AS `copyright`';
        }
    }

    /**
     * Adding copyright notice on product page
     *
     * @return mixed
     * @throws PrestaShopException
     */
    public function hookExtraRight()
    {
        if ($copyright_product = CopyrightProduct::getProduct(Tools::getValue('id_product'))) {
            $copyright_settings = new CopyrightSettings(CopyrightSettings::getCurrentIdCopyrightSettings());
            $copyright_notice = $copyright_settings->copyright_notice[$this->context->language->id];

            return $this->getCopyrightCertificateLink($copyright_notice, $copyright_product['id_copyright_file']);
        }
    }

    public function hookDisplayBackOfficeHeader()
    {

        $copyright_user = CopyrightUser::getCopyrightUser();
        $copyright_settings = new CopyrightSettings(CopyrightSettings::getCurrentIdCopyrightSettings());

        //adding icon image for admin tab
        $this->context->controller->addCSS($this->_path . 'views/css/menuTabIcon.css');

        $this->context->smarty->assign(array(
            'copyright_auth_url' => $this->context->link->getAdminLink('AdminCopyrightSettingsAuth'),
            'copyright_register_url' => $this->context->link->getAdminLink('AdminCopyrightSettingsRegister'),
            'copyright_auth_success_message' => $this->l("The authorization is successful."),
            'copyright_registration_success_message' => $this->l("The registration is successful."),

            //for the credit count indicator in the left side menu
            'copyright_user' => Tools::jsonEncode($copyright_user),
            'used_copyright_credits_count' => $copyright_user ? $copyright_user->getUsedCreditsCount() : 0,
            'copyright_settings' => Tools::jsonEncode($copyright_settings),
            'id_history_tab' => $this->getParentTabMenuId(),
        ));
        
        return $this->display(__FILE__, 'displayBackOfficeHeader.smarty');
    }

    public function hookDisplayBackOfficeTop()
    {
        $this->context->smarty->assign(array(
            'link' => $this->context->link,
            'copyright_user' => CopyrightUser::getCopyrightUser(),
            'copyright_settings' => new CopyrightSettings(CopyrightSettings::getCurrentIdCopyrightSettings()),
        ));

        return $this->display(__FILE__, 'displayBackOfficeTop.tpl');
    }

    public function hookActionAdminCmsFormModifier($params)
    {
        $fields = &$params['fields'][0]['form']['input'];
        $fields_value = &$params['fields_value'];
        $id_cms = Tools::getValue('id_cms', 0);
        $copyright_user = CopyrightUser::getCopyrightUser();

        //validate access to create a copyright
        $errors = $this->validate('cms');

        if (empty($errors)) {
            $fields[] = array(
                'type' => 'html',
                'name' => $this->renderInfo(
                    $copyright_user->copyright_credits
                    . ' '
                    . $this->l('copyright(s) available for this content.')
                ),
            );

            if (CopyrightCms::getRegisterCms($id_cms)) {
                $fields[] = array(
                    'type' => 'html',
                    'name' => $this->renderSuccess($this->l('The cms is registered.')),
                );
            } elseif (CopyrightCms::inProgress($id_cms)) {
                $fields[] = array(
                    'type' => 'html',
                    'name' => $this->renderWarning($this->l('Processing the cms in.')),
                );
            } else {
                $fields[] = array(
                    'type' => 'switch',
                    'label' => $this->l('Copyright registration'),
                    'name' => 'copyright_registration',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'active_on',
                            'value' => 1,
                            'label' => $this->l('Enabled')
                        ),
                        array(
                            'id' => 'active_off',
                            'value' => 0,
                            'label' => $this->l('Disabled')
                        )
                    ),
                );

                $fields_value['copyright_registration'] = 0;
            }

        } else {
            
            $fields[] = array(
                'type' => 'html',
                'name' => $this->renderErrors($errors),
            );
        }

    }

    public function hookActionAdminCategoriesFormModifier($params)
    {
        $fields = &$params['fields'][0]['form']['input'];
        $fields_value = &$params['fields_value'];
        $id_category = Tools::getValue('id_category', 0);
        $copyright_user = CopyrightUser::getCopyrightUser();

        //validate access to create a copyright
        $errors = $this->validate();

        if (empty($errors)) {
            $fields[] = array(
                'type' => 'html',
                'name' => $this->renderInfo(
                    $copyright_user->copyright_credits
                    . ' '
                    . $this->l('copyright(s) available for this content.')
                ),
            );

            if (CopyrightCategory::getRegisterCategory($id_category)) {
                $fields[] = array(
                    'type' => 'html',
                    'name' => $this->renderSuccess($this->l('The category is registered.')),
                );
            } elseif (CopyrightCategory::inProgress($id_category)) {
                $fields[] = array(
                    'type' => 'html',
                    'name' => $this->renderWarning($this->l('Processing the category in.')),
                );
            } else {

                $fields[] = array(
                    'type' => 'switch',
                    'label' => $this->l('Copyright registration'),
                    'name' => 'copyright_registration',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'active_on',
                            'value' => 1,
                            'label' => $this->l('Enabled')
                        ),
                        array(
                            'id' => 'active_off',
                            'value' => 0,
                            'label' => $this->l('Disabled')
                        )
                    ),
                );

                $fields_value['copyright_registration'] = 0;
            }

        } else {

            $fields[] = array(
                'type' => 'html',
                'name' => $this->renderErrors($errors),
            );
        }

    }

    public function hookHeader()
    {
        $controller = $this->context->controller;
        $id_copyright_settings = CopyrightSettings::getCurrentIdCopyrightSettings();
        $copyright_settings = new CopyrightSettings($id_copyright_settings, $this->context->language->id);
        $copyright_notice = $copyright_settings->copyright_notice;

        if ($controller->php_self == 'cms'
            && $cms = CopyrightCms::getRegisterCms(Tools::getValue('id_cms'))) {
            $id_copyright_file = $cms['id_copyright_file'];
            Media::addJsDef(array(
                'copyright_notice' => $this->getCopyrightCertificateLink($copyright_notice, $id_copyright_file),
            ));
            $controller->addJS($this->_path . 'views/js/cms_copyright.js');
            
        } elseif ($controller->php_self == 'category'
            && $category = CopyrightCategory::getRegisterCategory(Tools::getValue('id_category'))) {
            $id_copyright_file = $category['id_copyright_file'];
            Media::addJsDef(array(
                'copyright_notice' => $this->getCopyrightCertificateLink($copyright_notice, $id_copyright_file),
            ));
            $controller->addJS($this->_path . 'views/js/category_copyright.js');
        }
    }

    public function renderErrors($errors = array())
    {
        return $this->renderTemplate('errors.smarty', 'admin', array('errors' => $errors));
    }

    public function renderInfo($infos)
    {
        if (!is_array($infos)) {
            $infos = array($infos);
        }

        return $this->renderTemplate('infos.smarty', 'admin', array('infos' => $infos));
    }

    public function renderWarning($warning)
    {
        if (!is_array($warning)) {
            $warning = array($warning);
        }

        return $this->renderTemplate('warning.smarty', 'admin', array('warnings' => $warning));
    }

    public function renderSuccess($success)
    {
        if (!is_array($success)) {
            $success = array($success);
        }

        return $this->renderTemplate('success.tpl', 'admin', array('successes' => $success));
    }

    public function getCopyrightCertificateLink($copyright_notice = '', $id_copyright_file = 0)
    {
        if (preg_match('/\{\s*certificate\s*=\s*"(.*)"\}/', $copyright_notice, $copyright_link_text)) {
            if (isset($copyright_link_text[0]) && isset($copyright_link_text[1])) {
                $copyright_file = new CopyrightFile($id_copyright_file);
                $copyright_notice_link = '<a class="copyright-certificate-link" href="'
                    . $copyright_file->certificate_url . '">'
                    . $copyright_link_text[1] . '</a>';

                return str_replace($copyright_link_text[0], $copyright_notice_link, $copyright_notice);
            }
        }
    }

    public function processAll() {

        //register products
        foreach (CopyrightProduct::getNotUpdatedProducts() as $product) {
            if (!CopyrightProduct::createProductCopyright($product['id_product'])) {
                $this->errors[] = $this->l(
                    "Copyright: id_product " . $product['id_product'] . " doesn't create copyright."
                );
            }
        }

        //register category
        foreach (CopyrightCategory::getNotUpdatedCategory() as $category) {
            if (!CopyrightCategory::createCategoryCopyright($category['id_category'])) {
                $this->errors[] = $this->l(
                    "Copyright: id_category " . $category['id_category'] . " doesn't create copyright."
                );
            }
        }

        //register cms
        foreach (CopyrightCms::getNotUpdatedCms() as $cms) {
            if (!CopyrightCms::createCmsCopyright($cms['id_cms'])) {
                $this->errors[] = $this->l(
                    "Copyright: id_cms " . $cms['id_cms'] . " doesn't create copyright."
                );
            }
        }

        //$copyright = Module::getInstanceByName('copyright');
        $this->checkHistory();

    }

    public function checkHistory()
    {
        //create a new user or get a old copyright user
        $copyright_user = CopyrightUser::getCopyrightUser();
        /* Add certificate link and document link from history */
        $copyright_items = array_merge(
            CopyrightProduct::getNotRegistrationProducts(),
            CopyrightCategory::getNotRegistrationCategory(),
            CopyrightCms::getNotRegistrationCms()
        );
        
        if (!empty($copyright_items)) {
            $history = CopyrightConnect::getInstance()->api_auth(
                '/api/Copyright/historic?catalog=' . $copyright_user->cat
                . '&ID_client=' . $copyright_user->id_client
            );
            
            foreach ($copyright_items as $copyright_item) {
                foreach ($history as $copyright) {
                    if ($copyright_item['id_deposit'] == $copyright['ID_deposit']) {
                        $copyright_file = new CopyrightFile($copyright_item['id_copyright_file']);
                        $copyright_file->certificate_url = $copyright['CertificateUrl'];
                        $copyright_file->registered_file_url = $copyright['RegisteredFileUrl'];
                        $copyright_file->type = $copyright['Type'];
                        $copyright_file->status = 1;
                        $copyright_file->save();
                    }
                }
            }
        }
    }

    /**
     * Add new page in back office
     *
     * @param type $class_name
     * @param type $tab_name
     * @param type $id_parent
     * @param type $position
     *
     * @return type
     */
    public function installModuleTab($class_name, $tab_name, $id_parent = 0, $position = 0, $active = true)
    {

        $tab = new Tab();
        $tab->active = $active;
        $tab->class_name = $class_name;
        $tab->name = array();

        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = $tab_name;
        }

        $tab->id_parent = $id_parent;
        $tab->position = $position;
        $tab->module = $this->name;

        return $tab->save();
    }

    /**
     * Delete custom page of back office
     *
     * @param type $class_name
     *
     * @return type
     */
    public function uninstallModuleTab($class_name)
    {

        $id_tab = Tab::getIdFromClassName($class_name);

        if ($id_tab) {

            $tab = new Tab($id_tab);
            $tab->delete();
            return true;
        }

        return false;
    }

    public function uninstallModuleTabs()
    {
        return Db::getInstance()->delete('tab', 'module = "' . pSQL($this->name) . '"');
    }

    public function getParentTabMenuId()
    {
        return Db::getInstance()->getValue(
            'SELECT id_tab FROM ' . _DB_PREFIX_ . 'tab 
                WHERE module = "' . pSQL($this->name) . '" 
                        AND id_parent = 0;'
        );
    }

    public function renderTemplate($tpl, $type, $vars = array())
    {
        $tpl = COPYRIGHT_DIR . 'views/templates/' . $type . '/' . $tpl;
        if (!file_exists($tpl)) {
            $this->context->controller->errors[] = $this->l("File " . $tpl . " doesn't found.");
            return false;
        }

        if (!is_array($vars)) {
            $vars = array($vars);
        }

        $about = $this->context->smarty->createTemplate($tpl);

        $about->assign($vars);

        return $about->fetch();
    }

    public function getViewUri()
    {
        return Tools::getHttpHost(true) . $this->getPathUri() . 'views/';
    }
}
